@@ -3,7 +3,7 @@ import importlib
 import importlib.util
 
 from typing import Callable, Optional, Union
-from types import FunctionType, ModuleType
+from types import FunctionType, ModuleType, BuiltinFunctionType
 from multipledispatch import dispatch
 
 from .utils import get_sub_dict
@@ -19,6 +19,7 @@ class Constructor:
         self.ignore_rewriting = ignore_rewriting
         if load_builders:
             self.load_builders()
+        self.vars = {}
 
     def construct(self, element):
         return element.construct(self)
@@ -45,7 +46,15 @@ class Constructor:
 
 
 class ConstructorError(Exception):
-    pass
+    def __init__(self, element, args, kwargs):
+        self.cls = type(element).__name__
+        self.name = element.name
+        self.args = args
+        self.kwargs = kwargs
+
+    def __str__(self):
+        return f"Unable to construct {self.cls} '{self.name}' with args:{self.args} and " \
+               f"kwargs:{self.kwargs}"
 
 
 # mb: add meta for auto detecting this class as YAP-builder
@@ -78,14 +87,14 @@ def nip(wrap_call=False):
     return nip_decorator(wrap_call=wrap_call)
 
 
-@dispatch([(type, FunctionType)])
+@dispatch([(type, FunctionType, BuiltinFunctionType)])
 def nip(item: Union[type, FunctionType]):
     return nip_decorator()(item)
 
 
 @dispatch(ModuleType)
-def nip(module):
-    wrap_module(module)
+def nip(module, wrap_builtins=False):
+    wrap_module(module, wrap_builtins)
 
 
 def call_wrapper(item: Union[type, FunctionType]):  # wraps call for convenient object dump
@@ -104,7 +113,7 @@ def call_wrapper(item: Union[type, FunctionType]):  # wraps call for convenient
     return call_dumper
 
 
-def wrap_module(module: Union[str, ModuleType]):
+def wrap_module(module: Union[str, ModuleType], wrap_builtins=False):
     """Wraps everything declared in module with @nip
 
     Parameters
@@ -116,5 +125,6 @@ def wrap_module(module: Union[str, ModuleType]):
         module = importlib.import_module(module)
 
     for value in module.__dict__.values():
-        if isinstance(value, (type, FunctionType)):
+        if isinstance(value, (type, FunctionType)) or \
+                wrap_builtins and isinstance(value, BuiltinFunctionType):
             nip(value)
